@@ -12,6 +12,14 @@ pub enum OutputFormat {
     Json,
     /// No output -- meant to be used for testing
     None,
+    /// Save all versions tested and save success result for all runs -- meant to be used for testing
+    TestSuccesses,
+}
+
+impl Default for OutputFormat {
+    fn default() -> Self {
+        Self::Human
+    }
 }
 
 /// Gets a [`Config`] from the given matches, but sets output_format to None
@@ -40,6 +48,24 @@ impl From<ModeIntent> for &'static str {
     }
 }
 
+#[derive(Debug, Clone, Copy)]
+pub enum ReleaseSource {
+    RustChangelog,
+    RustDist,
+}
+
+impl TryFrom<&str> for ReleaseSource {
+    type Error = CargoMSRVError;
+
+    fn try_from(source: &str) -> Result<Self, Self::Error> {
+        match source {
+            "rust-changelog" => Ok(Self::RustChangelog),
+            "rust-dist" => Ok(Self::RustDist),
+            s => Err(CargoMSRVError::RustReleasesSourceParseError(s.to_string())),
+        }
+    }
+}
+
 #[derive(Debug, Clone)]
 pub struct Config<'a> {
     mode_intent: ModeIntent,
@@ -53,6 +79,9 @@ pub struct Config<'a> {
     output_toolchain_file: bool,
     ignore_lockfile: bool,
     output_format: OutputFormat,
+    release_source: ReleaseSource,
+    no_tracing: bool,
+    no_read_min_edition: Option<semver::Version>,
 }
 
 impl<'a> Config<'a> {
@@ -69,6 +98,9 @@ impl<'a> Config<'a> {
             output_toolchain_file: false,
             ignore_lockfile: false,
             output_format: OutputFormat::Human,
+            release_source: ReleaseSource::RustChangelog,
+            no_tracing: false,
+            no_read_min_edition: None,
         }
     }
 
@@ -84,6 +116,10 @@ impl<'a> Config<'a> {
         &self.check_command
     }
 
+    pub fn check_command_string(&self) -> String {
+        self.check_command.join(" ")
+    }
+
     pub fn crate_path(&self) -> Option<&Path> {
         self.crate_path.as_deref()
     }
@@ -115,6 +151,18 @@ impl<'a> Config<'a> {
     pub fn output_format(&self) -> OutputFormat {
         self.output_format
     }
+
+    pub fn release_source(&self) -> ReleaseSource {
+        self.release_source
+    }
+
+    pub fn no_tracing(&self) -> bool {
+        self.no_tracing
+    }
+
+    pub fn no_read_min_version(&self) -> Option<&semver::Version> {
+        self.no_read_min_edition.as_ref()
+    }
 }
 
 #[derive(Debug, Clone)]
@@ -184,6 +232,21 @@ impl<'a> ConfigBuilder<'a> {
         self
     }
 
+    pub fn release_source(mut self, release_source: ReleaseSource) -> Self {
+        self.inner.release_source = release_source;
+        self
+    }
+
+    pub fn no_tracing(mut self, choice: bool) -> Self {
+        self.inner.no_tracing = choice;
+        self
+    }
+
+    pub fn no_read_min_edition(mut self, version: semver::Version) -> Self {
+        self.inner.no_read_min_edition = Some(version);
+        self
+    }
+
     pub fn build(self) -> Config<'a> {
         self.inner
     }
@@ -227,8 +290,29 @@ impl<'config> TryFrom<&'config ArgMatches<'config>> for Config<'config> {
             builder = builder.target(target);
         }
 
-        if let Some(min) = arg_matches.value_of(id::ARG_MIN) {
-            builder = builder.minimum_version(parse_version(min)?)
+        match arg_matches.value_of(id::ARG_MIN) {
+            Some(min) => builder = builder.minimum_version(parse_version(min)?),
+            None if arg_matches.is_present(id::ARG_NO_READ_MIN_EDITION) => {}
+            None => {
+                let crate_folder = if let Some(ref path) = builder.inner.crate_path {
+                    Ok(path.to_path_buf())
+                } else {
+                    std::env::current_dir().map_err(CargoMSRVError::Io)
+                }?;
+                let cargo_toml = crate_folder.join("Cargo.toml");
+
+                let contents = std::fs::read_to_string(&cargo_toml).map_err(CargoMSRVError::Io)?;
+                let document = decent_toml_rs_alternative::parse_toml(&contents)
+                    .map_err(CargoMSRVError::ParseToml)?;
+
+                if let Some(edition) = document
+                    .get("package")
+                    .and_then(|field| field.get("edition"))
+                    .and_then(|value| value.as_string())
+                {
+                    builder = builder.minimum_version(parse_version(edition.as_str())?)
+                }
+            }
         }
 
         if let Some(max) = arg_matches.value_of(id::ARG_MAX) {
@@ -254,6 +338,14 @@ impl<'config> TryFrom<&'config ArgMatches<'config>> for Config<'config> {
             builder = builder.output_format(output_format);
         }
 
+        let release_source = arg_matches.value_of(id::ARG_RELEASE_SOURCE);
+        if let Some(release_source) = release_source {
+            let release_source = ReleaseSource::try_from(release_source)?;
+            builder = builder.release_source(release_source);
+        }
+
+        builder = builder.no_tracing(arg_matches.is_present(id::ARG_NO_LOG));
+
         Ok(builder.build())
     }
 }
