@@ -1,15 +1,17 @@
 #![deny(clippy::all)]
 #![allow(clippy::upper_case_acronyms, clippy::unnecessary_wraps)]
 
-use crate::check::{as_toolchain_specifier, check_toolchain, CheckStatus};
-use crate::config::{Config, ModeIntent};
+use crate::check::{as_toolchain_specifier, check_toolchain, Outcome};
+use crate::config::{Config, ModeIntent, ReleaseSource};
 use crate::errors::{CargoMSRVError, TResult};
-use crate::reporter::{json, ui};
-use crate::reporter::{Output, ProgressAction, __private::NoOutput};
+use crate::manifest::{CargoManifest, CargoManifestParser, TomlMap, TomlParser};
+use crate::reporter::Reporter;
+use crate::reporter::{Output, ProgressAction};
 use rust_releases::linear::LatestStableReleases;
 use rust_releases::{
-    semver, Channel, FetchResources, Release, ReleaseIndex, RustChangelog, Source,
+    semver, Channel, FetchResources, Release, ReleaseIndex, RustChangelog, RustDist, Source,
 };
+use std::convert::TryFrom;
 use std::path::PathBuf;
 
 pub mod check;
@@ -19,20 +21,31 @@ pub mod config;
 pub mod errors;
 pub mod fetch;
 pub mod lockfile;
+pub(crate) mod manifest;
 pub mod reporter;
 
-pub fn run_app(config: &Config) -> TResult<()> {
-    let index_strategy = RustChangelog::fetch_channel(Channel::Stable)?;
-    let index = index_strategy.build_index()?;
+pub fn run_app(config: &Config, reporter: &Reporter) -> TResult<()> {
+    reporter.progress(ProgressAction::FetchingIndex);
+
+    let index = match config.release_source() {
+        ReleaseSource::RustChangelog => {
+            RustChangelog::fetch_channel(Channel::Stable)?.build_index()?
+        }
+        ReleaseSource::RustDist => RustDist::fetch_channel(Channel::Stable)?.build_index()?,
+    };
 
     match config.action_intent() {
-        ModeIntent::DetermineMSRV => run_determine_msrv_action(config, &index),
-        ModeIntent::VerifyMSRV => run_verify_msrv_action(config, &index),
+        ModeIntent::DetermineMSRV => run_determine_msrv_action(config, reporter, &index),
+        ModeIntent::VerifyMSRV => run_verify_msrv_action(config, reporter, &index),
     }
 }
 
-fn run_determine_msrv_action(config: &Config, release_index: &ReleaseIndex) -> TResult<()> {
-    match determine_msrv(config, release_index)? {
+fn run_determine_msrv_action(
+    config: &Config,
+    reporter: &Reporter,
+    release_index: &ReleaseIndex,
+) -> TResult<()> {
+    match determine_msrv(config, reporter, release_index)? {
         MinimalCompatibility::NoCompatibleToolchains => {
             Err(CargoMSRVError::UnableToFindAnyGoodVersion {
                 command: config.check_command().join(" "),
@@ -47,55 +60,43 @@ fn run_determine_msrv_action(config: &Config, release_index: &ReleaseIndex) -> T
     }
 }
 
-fn run_verify_msrv_action(config: &Config, _release_index: &ReleaseIndex) -> TResult<()> {
+// NB: only public for integration testing
+pub fn run_verify_msrv_action(
+    config: &Config,
+    reporter: &Reporter,
+    release_index: &ReleaseIndex,
+) -> TResult<()> {
     let crate_folder = crate_root_folder(config)?;
     let cargo_toml = crate_folder.join("Cargo.toml");
 
     let contents = std::fs::read_to_string(&cargo_toml).map_err(CargoMSRVError::Io)?;
-    let document =
-        decent_toml_rs_alternative::parse_toml(&contents).map_err(CargoMSRVError::ParseToml)?;
 
-    let msrv = document
-        .get("package")
-        .and_then(|field| field.get("metadata"))
-        .and_then(|field| field.get("msrv"))
-        .and_then(|value| value.as_string())
+    let manifest = CargoManifestParser::default().parse::<TomlMap>(&contents)?;
+    let manifest = CargoManifest::try_from(manifest)?;
+
+    let version = manifest
+        .minimum_rust_version()
         .ok_or(CargoMSRVError::NoMSRVKeyInCargoToml(cargo_toml))?;
+    let version = version.try_to_semver(
+        release_index
+            .releases()
+            .iter()
+            .map(|release| release.version()),
+    )?;
 
-    let version = semver::Version::parse(&msrv).map_err(CargoMSRVError::SemverError)?;
-
-    let cmd = config.check_command().join(" ");
-
-    match config.output_format() {
-        config::OutputFormat::Human => {
-            let reporter = ui::HumanPrinter::new(1, config.target(), &cmd);
-            reporter.mode(ModeIntent::VerifyMSRV);
-            let status = check_toolchain(&version, config, &reporter)?;
-            report_verify_completion(&reporter, status, &cmd);
-        }
-        config::OutputFormat::Json => {
-            let reporter = json::JsonPrinter::new(1, config.target(), &cmd);
-            reporter.mode(ModeIntent::VerifyMSRV);
-            let status = check_toolchain(&version, config, &reporter)?;
-            report_verify_completion(&reporter, status, &cmd);
-        }
-        config::OutputFormat::None => {
-            let reporter = NoOutput;
-            reporter.mode(ModeIntent::VerifyMSRV);
-            let status = check_toolchain(&version, config, &reporter)?;
-            report_verify_completion(&reporter, status, &cmd);
-        }
-    };
+    let cmd = config.check_command_string();
+    reporter.mode(ModeIntent::VerifyMSRV);
+    let status = check_toolchain(version, config, reporter)?;
+    report_verify_completion(reporter, status, &cmd);
 
     Ok(())
 }
 
-fn report_verify_completion(output: &impl Output, status: CheckStatus, cmd: &str) {
-    match status {
-        CheckStatus::Success { version, .. } => {
-            output.finish_success(ModeIntent::VerifyMSRV, &version)
-        }
-        CheckStatus::Failure { .. } => output.finish_failure(ModeIntent::VerifyMSRV, cmd),
+fn report_verify_completion(output: &impl Output, status: Outcome, cmd: &str) {
+    if status.is_success() {
+        output.finish_success(ModeIntent::VerifyMSRV, status.version());
+    } else {
+        output.finish_failure(ModeIntent::VerifyMSRV, cmd);
     }
 }
 
@@ -123,24 +124,25 @@ impl MinimalCompatibility {
     }
 }
 
-impl From<CheckStatus> for MinimalCompatibility {
-    fn from(from: CheckStatus) -> Self {
-        match from {
-            CheckStatus::Success { version, toolchain } => {
-                MinimalCompatibility::CapableToolchain { version, toolchain }
-            }
-            CheckStatus::Failure { toolchain: _, .. } => {
-                MinimalCompatibility::NoCompatibleToolchains
-            }
+impl From<Outcome> for MinimalCompatibility {
+    fn from(outcome: Outcome) -> Self {
+        let version = outcome.version().to_owned();
+        let toolchain = outcome.toolchain().to_string();
+
+        if outcome.is_success() {
+            MinimalCompatibility::CapableToolchain { version, toolchain }
+        } else {
+            MinimalCompatibility::NoCompatibleToolchains
         }
     }
 }
 
 pub fn determine_msrv(
     config: &Config,
+    reporter: &Reporter,
     index: &rust_releases::ReleaseIndex,
 ) -> TResult<MinimalCompatibility> {
-    let cmd = config.check_command().join(" ");
+    let cmd = config.check_command_string();
 
     let releases = index.releases();
 
@@ -166,24 +168,9 @@ pub fn determine_msrv(
         })
         .collect::<Vec<_>>();
 
-    match config.output_format() {
-        config::OutputFormat::Human => {
-            let ui = ui::HumanPrinter::new(included_releases.len() as u64, config.target(), &cmd);
-            ui.mode(ModeIntent::DetermineMSRV);
-            determine_msrv_impl(config, &included_releases, &cmd, &ui)
-        }
-        config::OutputFormat::Json => {
-            let output =
-                json::JsonPrinter::new(included_releases.len() as u64, config.target(), &cmd);
-            output.mode(ModeIntent::DetermineMSRV);
-            determine_msrv_impl(config, &included_releases, &cmd, &output)
-        }
-        config::OutputFormat::None => {
-            let output = NoOutput;
-            output.mode(ModeIntent::DetermineMSRV);
-            determine_msrv_impl(config, &included_releases, &cmd, &output)
-        }
-    }
+    reporter.mode(ModeIntent::DetermineMSRV);
+    reporter.set_steps(included_releases.len() as u64);
+    determine_msrv_impl(config, &included_releases, &cmd, reporter)
 }
 
 fn determine_msrv_impl(
@@ -225,14 +212,14 @@ fn test_against_releases_linearly(
     output: &impl Output,
 ) -> TResult<()> {
     for release in releases {
-        output.progress(ProgressAction::Checking, release.version());
-        let status = check_toolchain(release.version(), config, output)?;
+        output.progress(ProgressAction::Checking(release.version()));
+        let outcome = check_toolchain(release.version(), config, output)?;
 
-        if let CheckStatus::Failure { .. } = status {
+        if !outcome.is_success() {
             break;
         }
 
-        *compatibility = status.into();
+        *compatibility = outcome.into();
     }
 
     Ok(())
@@ -251,17 +238,20 @@ fn test_against_releases_bisect(
     let progressed = std::cell::Cell::new(0u64);
     let mut binary_search = Bisect::from_slice(releases);
     let outcome = binary_search.search_with_result_and_remainder(|release, remainder| {
-        output.progress(ProgressAction::Checking, release.version());
+        output.progress(ProgressAction::Checking(release.version()));
 
         // increment progressed items
         let steps = progressed.replace(progressed.get().saturating_add(1));
         output.set_steps(steps + (remainder as u64));
 
-        let status = check_toolchain(release.version(), config, output)?;
+        let outcome = check_toolchain(release.version(), config, output)?;
 
-        match status {
-            CheckStatus::Failure { .. } => TResult::Ok(Narrow::ToLeft),
-            CheckStatus::Success { .. } => TResult::Ok(Narrow::ToRight),
+        if outcome.is_success() {
+            // Expand search space
+            TResult::Ok(Narrow::ToRight)
+        } else {
+            // Shrink search space
+            TResult::Ok(Narrow::ToLeft)
         }
     });
 
